module Mahoujin
  module Graphics
    class Stringify
      def stringify(atom)
        stringify_atom(atom)
      end

      private

      def stringify_atom(atom)
        send "stringify_#{atom.class.underscore_name}", atom
      end

      def stringify_syntax(atom)
        atom.atoms.map(&->(a) { stringify_atom(a) }).join("\n")
      end

      def stringify_rule(atom)
        "#{stringify_atom(atom.rule_name_atom)} = #{stringify_atom(atom.atom)};"
      end

      def stringify_rule_name(atom)
        atom.content
      end

      def stringify_choice(atom)
        atom.atoms.map(&->(a) { stringify_atom(a) }).join(' | ')
      end

      def stringify_concatenation(atom)
        atom.atoms.map(&->(a) { stringify_atom(a) }).join(', ')
      end

      def stringify_specific_repetition(atom)
        "#{atom.number} * #{stringify_atom(atom.atom)}"
      end

      def stringify_optional(atom)
        "[ #{stringify_atom(atom.atom)} ]"
      end

      def stringify_zero_or_more_repetition(atom)
        "{ #{stringify_atom(atom.atom)} }"
      end

      def stringify_grouping(atom)
        "( #{stringify_atom(atom.atom)} )"
      end

      def stringify_prose(atom)
        "? #{atom.content} ?"
      end

      def stringify_exception(atom)
        atom.content
      end

      def stringify_non_terminal(atom)
        atom.content
      end

      def stringify_terminal(atom)
        atom.content.include?("'") ? "\"#{atom.content}\"" : "'#{atom.content}'"
      end
    end
  end
end
