module Mahoujin
  module Graphics
    module Styles
      class Json
        attr_reader :basic

        def initialize
          initialize_basic
          initialize_atoms
        end

        def query(atom, attribute)
          @atoms.fetch(atom).fetch(attribute)
        end

        private

        def initialize_basic
          @basic = {}
          @basic[:font]  = { family: 'serif', size: 12 }.freeze
          @basic[:line]  = { width: 3 }.freeze

          @basic[:unit] = nil
          @basic[:corner] = { radius: 0.618 }.freeze
        end

        def initialize_atoms
          @atoms = {}

          @atoms[Atoms::Terminal]    = { bsize: { width: 2 * 0.618, height: 1 + 2 * 0.618 }.freeze, font: {                          }.freeze, background: '#CEFFCE' }
          @atoms[Atoms::NonTerminal] = { bsize: { width: 2 * 0.618, height: 1 + 2 * 0.618 }.freeze, font: { bold: true, italic: true }.freeze, background: '#CEFFCE' }
          @atoms[Atoms::Prose]       = { bsize: { width: 2 * 0.618, height: 1 + 2 * 0.618 }.freeze, font: {             italic: true }.freeze, background: '#CEFFCE' }
          @atoms[Atoms::Exception]   = { bsize: { width: 2 * 0.618, height: 1 + 2 * 0.618 }.freeze, font: {             italic: true }.freeze, background: '#CEFFCE' }
          @atoms[Atoms::RuleName]    = { bsize: { width: 0        , height: 1             }.freeze, font: { bold: true, italic: true }.freeze }

          @atoms[Atoms::ZeroOrMoreRepetition] = { padding: { top: 1.5, bottom: 1.5, left: 4.5, right: 4.5 }.freeze }
          @atoms[Atoms::Optional]             = { padding: {           bottom: 1.5, left: 3.0, right: 3.0 }.freeze }
          @atoms[Atoms::SpecificRepetition]   = { padding: { top: 1.5,              left: 1.5, right: 1.5 }.freeze }

          @atoms[Atoms::Concatenation] = { space: 1.5 }
          @atoms[Atoms::Choice]        = { space: 1, padding: {                    left: 3, right: 3 }.freeze }
          @atoms[Atoms::Rule]          = { space: 2, padding: {                    left: 2, right: 2 }.freeze }
          @atoms[Atoms::Syntax]        = { space: 4, padding: { top: 2, bottom: 2, left: 2, right: 2 }.freeze }
        end
      end
    end
  end
end
