module Mahoujin
  class Parser < Parslet::Parser
    root(:_start_)
    rule(:_start_) { syntax.as(:syntax) }

    rule(:syntax) do
      space? >> syntax_rule.as(:syntax_rule) >>
        (space? >> syntax_rule.as(:syntax_rule)).repeat >> space?
    end

    rule(:syntax_rule) do
      meta_identifier.as(:meta_identifier) >> space? >> str('=') >>
        space? >> definitions_list.as(:definitions_list) >> space? >> str(';')
    end

    rule(:definitions_list) do
      single_definition.as(:single_definition) >>
        (space? >> str('|') >> space? >> single_definition.as(:single_definition)).repeat
    end

    rule(:single_definition) do
      term.as(:term) >>
        (space? >> str(',') >> space? >> term.as(:term)).repeat
    end

    rule(:term) do
      factor.as(:factor) >>
        (space? >> str('-') >> space? >> exception.as(:exception)).maybe
    end

    rule(:exception) do
      factor
    end

    rule(:factor) do
      (integer.as(:number_of_repetition) >> space? >> str('*') >> space?).maybe >>
        primary.as(:primary)
    end

    rule(:primary) do
      optional_sequence.as(:optional_sequence) |
        repeated_sequence.as(:repeated_sequence) |
        grouped_sequence.as(:grouped_sequence) |
        special_sequence.as(:special_sequence) |
        meta_identifier.as(:meta_identifier) |
        terminal_string.as(:terminal_string)
    end

    rule(:optional_sequence)   { str('[') >> space? >> definitions_list.as(:definitions_list) >> space? >> str(']') }
    rule(:repeated_sequence)   { str('{') >> space? >> definitions_list.as(:definitions_list) >> space? >> str('}') }
    rule(:grouped_sequence)    { str('(') >> space? >> definitions_list.as(:definitions_list) >> space? >> str(')') }
    rule(:special_sequence)    { str('?') >> match('[^?]').repeat >> str('?') }
    rule(:meta_identifier)     { match('[a-zA-Z]') >> match('[a-zA-Z0-9 ]').repeat }
    rule(:terminal_string)     { single_quote_string | double_quote_string }
    rule(:single_quote_string) { str("'") >> match("[^']").repeat(1) >> str("'") }
    rule(:double_quote_string) { str('"') >> match('[^"]').repeat(1) >> str('"') }
    rule(:integer)             { match('[0-9]').repeat(1) }
    rule(:space?)              { (whitespace | comment).repeat }
    rule(:whitespace)          { match('\s') }
    rule(:comment)             { str('(*') >> (str('*)').absent? >> any).repeat >> str('*)') }
  end
end
