module Mahoujin
  class Transform < Parslet::Transform
    rule(  terminal_string: simple(:token)) { Atoms::Terminal.new(token.to_s[1...-1]) }
    rule(  meta_identifier: simple(:token)) { Atoms::NonTerminal.new(token.to_s.rstrip) }
    rule( special_sequence: simple(:token)) { Atoms::Prose.new(token.to_s[1...-1].strip) }
    rule( grouped_sequence: simple(:atom))  { Atoms::Grouping.new(atom) }
    rule(repeated_sequence: simple(:atom))  { Atoms::ZeroOrMoreRepetition.new(atom) }
    rule(optional_sequence: simple(:atom))  { Atoms::Optional.new(atom) }

    rule(primary: simple(:atom)) { atom }
    rule(number_of_repetition: simple(:token), primary: simple(:atom)) do
      Atoms::SpecificRepetition.new(atom, token.to_i)
    end

    rule(factor: simple(:atom)) { atom }
    rule(factor: simple(:atom), exception: simple(:atom_ex)) do
      Atoms::Exception.new("#{Transform.stringify(atom)} - #{Transform.stringify(atom_ex)}")
    end

    rule(term: simple(:atom)) { atom }
    rule(single_definition: simple(:atom)) { atom }
    rule(single_definition: sequence(:array)) { Atoms::Concatenation.new(array) }

    rule(definitions_list: simple(:atom)) { atom }
    rule(definitions_list: sequence(:array)) { Atoms::Choice.new(array) }

    rule(meta_identifier: simple(:token), definitions_list: simple(:atom)) { Atoms::Rule.new(Atoms::RuleName.new(token.to_s.rstrip), atom) }
    rule(meta_identifier: simple(:token), definitions_list: sequence(:array)) { Atoms::Rule.new(Atoms::RuleName.new(token.to_s.rstrip), Atoms::Choice.new(array)) }

    rule(syntax_rule: simple(:atom)) { atom }
    rule(syntax: simple(:atom)) { Atoms::Syntax.new(Array(atom)) }
    rule(syntax: sequence(:array)) { Atoms::Syntax.new(array) }

    def self.stringify(atom)
      @stringify ||= Graphics::Stringify.new
      @stringify.stringify(atom)
    end
  end
end
